=begin
= Υ󥯸⤦äȤץ饰((-$Id: disp_referrer.rb,v 1.11.2.2 2006/03/18 10:29:22 tadatadashi Exp $-))
ܸ꥽

== 
ƥʤΥ󥯡󥸥θ̤̾Υ󥯸βˤ
Ȥɽޤ󥸥θ̤ϡˤޤȤޤ

ǿɽǤϡ̾Υ󥯸ʳΥ󥯸򱣤ޤ

== 
(1.1.2.39)饳ɤΤۤȤɤʤᡢ
* 󥸥˴ؤư㤦
* ѻߤ줿ץ󤬤
* ץ̾ѹ줿
ȤߴޤŪWWW֥饦Ǥ褦ˤʤä
ޤΤǡƤߤޤ

Ǥ٤ȡ
* åˤɽ®줿((-긵ǤϡåȤʤ
  ˤ٤ơ1ʬ2/3ۤɡǿ3ʬ1/2ۤɤμ»֤
  ޤ-))εǽϻǰʤ顢󥿥ʤsecure=true
  ǤϻȤޤ
* 󥯸ִꥹȤˤʤURLŪñWWW֥饦ꥹȤɲä
  褦ˤʤä
* ִʸκǽ[]ǰϤޤ줿ʸ뤳Ȥˤäơ桼
  ƥ꡼ߤǤ褦ˤʤä((-tDiaryΤȤϰ㤤Ĥ
  URLϣĤΥƥ꡼ƤʤȤˤդ-))
* ŪWWW֥饦Ǥ褦ˤʤä
* disp_referrer.rb̵ƤȤ
* Uconv饤֥Nora饤֥꤬Фʤˡ̵Фʤʤ
  ư
Ȥޤ

== Ȥ
Υץ饰򥤥󥹥ȡ뤹뤳ȤǡǥեȤǤϡ
* ʬɽǡΥ󥯸פƥʡ󥸥󡢤
  ¾ˤޤȤɽ褦ˤʤޤִʸκǸ()
  ȥǥ롼פޤޤ󥸥󤫤Υ󥯤ϡ
  ̤ˤޤȤޤ
* ǿɽǡΥ󥯸פ˥ƥʤ両󥸥󤫤Υ
  󥯤ɽʤʤޤ
󥯸URLΥȥؤִϡtDiaryΤΥ󥯸ִꥹȤȤ


ץˤĤƤϲŪʥץϡtDiary
̤顢֥󥯸⤦äȶפ򥯥å뤳ȤǤޤ
ꤹˤϡ
  Insecure: can't modify hash (SecurityError)
Ȥ顼ФǽޤtDiaryǤξˤϡ
tDiary򿷤1.5.5.20030806ʹߤȤִܡפ鲿ѹ 
ˡOKפ򲡤Ȥǥ顼ǤǤ礦

󥯸ִꥹȤ䥪ץѹϡåǥ쥯ȥ
ˤ륭åեdisp_referrer2.cachedisp_referrer2.cache~
饰̤鹹ɬפޤΥץ饰̤
ѹܤˤĤƤϡѹ˥åι⤷ޤ

󥯸ϡʲΤ褦ʴʬवޤ

: ̾Υ󥯸(Υ󥯸)
  󥯸ִꥹȤˤƤϤޤURLΤʳΤΡ
  @options['disp_referrer2.unknown.divide']=falseξϡ󥯸ִ
  ꥹȤˤƤϤޤʤURL⤳˴ޤޤޤ

  ˡ󥯸ִꥹȤˤäִ줿ʸκǽ[]ǰϤ
  줿ʸ󤬤ϡ򥫥ƥ꡼Ȳᤷƥƥ꡼̤ɽ
  ʬޤεǽˤϡWWW֥饦̤Ѥ뤫
  tdiary.conf@options['disp_referrer2.normal.categorize']=falseˤ
  Υץѹˤϥå򹹿ɬפ
  ޤ

: ƥ
  URL /a/ antenna/ antenna. ʤɤʸ󤬴ޤޤ뤫ִʸˡ
  ƥ links ʤɤʸ󤬴ޤޤ󥯸Ǥξϡ
  @options['disp_referrer2.antenna.url']
  @options['disp_referrer2.antenna.title']ˤäѹǤޤ
  tdiary.confԽƤå򹹿ɬפޤ

: ¾
  󥯸ִꥹȤˤʤäURLǤޤĹURLϡtDiaryΤִ
  ꥹȤˤä̾Υ󥯸ʬवƤޤǽޤ

: 
  Υץ饰˴ޤޤ븡󥸥ΥꥹȤ˰פURLǤꥹ
  DispRef2Setup::Enginesˤޤޤ󥸥ǧʤ
  URLϡۤȤɤξ硢̾Υ󥯸˺äɽƤޤǤ
  Τ褦ʾϡURL
  ((<URL:http://tdiary-users.sourceforge.jp/cgi-bin/wiki.cgi?disp_referrer2.rb>))
  Τ餻ƤȺԤӤޤ

=== Ķ
ruby-1.6.71.8.0ưǧƤޤʳΥСRubyǤ
ư뤫⤷ޤ

tdiary-1.5.3-20030509ʹߤǻȤޤtDiary-1.5Ǥϡ
00default.rbbot?᥽åɤƤʤᡢ󥸥Υ
Фƥ󥯸ɽƤޤޤ

secure⡼ɤǤȤޤåˤ®Ǥޤ

mod_rubyǤưϺΤȤǧƤޤ

=== 󥹥ȡˡ
ΥեtDiarypluginǥ쥯ȥ˥ԡƤΥץ
κǿǤϡ
((<URL:http://zunda.freeshell.org/d/plugin/disp_referrer2.rb>))
ˤϤǤ

ޤNora饤֥꤬󥹥ȡ뤵ƤˤϡURLβHTML
פˡRubyɸźդCGI饤֥Nora饤֥
Ѥޤˤꡢ®٤㴳®ʤޤ((-긵ǻȤ
ʬɽˤ֤1ûʤޤ-))NoraˤĤƤξ
٤ϡ((<URL:http://raa.ruby-lang.org/list.rhtml?name=Nora>))򻲾Ȥ


=== ץ
Ǥ륪ץΰϡDispRef2Setup::Defaultsˤ
Υץkeyκǽˡdisp_referrer2.פɲä뤳
ǡtdiary.conf@optionskeyȤʤꡢtdiary.confǤ褦ˤ
ޤΥץΤDispRef2URL::Cached_options˵󤲤
ƤΤϡѹκݤ˥åιɬפǤ

ޤtDiary̤֥󥯸⤦äȶפ֤ȤWWW
饦Ǥܤ⤢ޤ

== ռ
Υץ饰ϡ
* UTF-8ʸEUCʸؤѴǽ
* θ󥸥̾ȤURL
* 󥸥ΥܥåȤΥ󥰤κݤ˥󥯸ɽʤǽ
MUTOH Masaodisp_referrer.rb饳ԡԽƻȤ碌Ƥ
Ƥޤ(󥸥ΥܥåȤ˴ؤ뵡ǽϸߤtDiaryΤˤȤ
ޤƤޤ)

ޤURL᤹뵡ǽΰRuby°cgi.rb饳ԡԽ
Ȥ碌ƤƤޤ

ˡ̾Υ󥯸[]ǰϤޤ줿ʸȤäƥƥʬ륢
ϡkazuhikoΤΤǤ

ͤ˴դޤ

== Todos
* secure=trueǥ󥯸ִꥹȤΥƥȥեɤǥ꥿򲡤ݤư
* parse_as_search®: hostnameΥå塩

== ˤĤ
Copyright (C) 2003 zunda <zunda at freeshell.org>

Please note that some methods in this plugin are written by other
authors as written in the comments.

Permission is granted for use, copying, modification, distribution, and
distribution of modified versions of this work under the terms of GPL
version 2 or later.
=end

=begin ChangeLog
See ../ChangeLog for changes after this.

* Mon Sep 29, 2003 zunda <zunda at freeshell.org>
- forgot to change arguments after changing initialize()
* Thu Sep 25, 2003 zunda <zunda at freeshell.org>
- name.untaint to eval name
* Thu Sep 25, 2003 zunda <zunda at freeshell.org>
- use to_native instead of to_euc
* Mon Sep 19, 2003 zunda <zunda at freeshell.org>
- disp_referrer2.rb,v 1.1.2.104 commited as disp_referrer.rb
* Mon Sep  1, 2003 zunda <zunda at freeshell.org>
- more strcit check for infoseek search enigne
* Wed Aug 27, 2003 zunda <zunda at freeshell.org>
- rd.yahoo, Searchalot, Hotbot added
* Tue Aug 12, 2003 zunda <zunda at freeshell.org>
- search engine list cleaned up
* Mon Aug 11, 2003 zunda <zunda at freeshell.org>
- instance_eval for e[2] in the search engine list
* Wed Aug  7, 2003 zunda <zunda at freeshell.org>
- WWW browser configuration interface
  - åιμ¤ˤ褦ˤޤWWW֥饦ִ
    ꥹȤäˤϥꥹȤκǽɲäޤ
  - secure=trueǤ¾Υ󥯸ꥹȤɽǤ褦ˤʤޤ
- Regexp generation for Wiki sites
* Wed Aug  6, 2003 zunda <zunda at freeshell.org>
- WWW browser configuration interface
  - ʥץȥ󥯸ִꥹȤθΨŪԽWWW֥饦
    褦ˤʤޤsecure=trueǤϰεǽϻȤޤ
* Sat Aug  2, 2003 zunda <zunda at freeshell.org>
- Second version
- basic functions re-implemented
  - ץ̿̾ʤޤޤפʥץäޤ
    tdiary.confԽƤϡǤ򤷤ʤƤ
  - Nora饤֥ȥåѤǹ®ޤ
  - 󥸥ΥꥹȤץ饰ǻĤ褦ˤʤޤ&;ޤม
    ʸ̤ФǤޤ
* Mon Feb 17, 2003 zunda <zunda at freeshell.org>
- First version
=end

# Message strings
Disp_referrer2_name = '󥯸⤦äȶ'.taint
Disp_referrer2_abstract = <<'_END'.taint
	ƥʤΥ󥯡󥸥θ̤
	̾Υ󥯸βˤޤȤɽޤ
	󥸥θ̤ϡˤޤȤޤ
_END
Disp_referrer2_with_Nora = <<'_END'.taint
<p>
	Nora饤֥ȤäƤޤΤǡɽ®ϤǤ
</p>
_END
Disp_referrer2_without_Nora = <<'_END'.taint
<p>
	ɽ®٤ˤʤϡ
	<a href="http://raa.ruby-lang.org/list.rhtml?name=Nora">Nora</a>
	饤֥򥤥󥹥ȡ뤷ƤߤƤ
</p>
_END
Disp_referrer2_scanned_urls = <<'_END'.taint
<p>ˤ%dĤURLåΥȥ꡼ɲäޤ</p>
_END
Disp_referrer2_updated_urls = <<'_END'.taint
<p>åΤ%dĤURLޤ</p>
_END
Disp_referrer2_cache_info = <<'_END'.taint
<p>
	ߡå礭%1$sХȡ
	%2$sĤURLå夵Ƥޤ
</p>
_END
Disp_referrer2_update_info = <<'_END'.taint
<p>
	<a href="%1$s">󥯸</a>פѹθˤ⡢
	åιɬפ⤷ޤ
</p>
_END
Disp_referrer2_move_to_refererlist = <<'_END'.taint
	¾Υ󥯸ִꥹȤԽ<a href="%s">ܤ</a>
_END
Disp_referrer2_move_to_config = <<'_END'.taint
	Ū<a href="%s">ܤ</a>
_END
Disp_referrer2_also_todayslink = <<'_END'.taint
	󥯸ִꥹȤϡ<a href="%s">󥯸</a>פԽǤޤ
_END
Disp_referrer2_antenna_label = 'ƥ'.taint
Disp_referrer2_unknown_label = '¾Υ󥯸'.taint
Disp_referrer2_search_label = ''.taint
Disp_referrer2_search_unknown_keyword = ''.taint
Disp_referrer2_cache_label = '(%sΥå)'.taint

class DispRef2SetupIF

	# show options
	def show_options
		r = <<-_HTML
			<h3>󥯸ʬɽ</h3>
			<table>
			<tr>
				<td><input name="dr2.current_mode" value="#{Options}" type="hidden">
				󥯸ִꥹȤˤʤ󥯸
				<td><input name="dr2.unknown.divide" value="true" type="radio"#{' checked'if @setup['unknown.divide']}>#{@setup['unknown.label']}Ȥʬ
				<td><input name="dr2.unknown.divide" value="false" type="radio"#{' checked'if not @setup['unknown.divide']}>̾Υ󥯸Ⱥ롣
			<tr>
				<td>#{@setup['unknown.label']}
				<td><input name="dr2.unknown.hide" value="false" type="radio"#{' checked'if not @setup['unknown.hide']}>ɽ
				<td><input name="dr2.unknown.hide" value="true" type="radio"#{' checked'if @setup['unknown.hide']}>
			<tr>
				<td>󥯸ִꥹȤִʸκǽ[]򥫥ƥ꡼ʬ
				<td><input name="dr2.normal.categorize" value="true" type="radio"#{' checked'if @setup['normal.categorize']}>Ȥ
				<td><input name="dr2.normal.categorize" value="false" type="radio"#{' checked'if not @setup['normal.categorize']}>Ȥʤ
			<tr>
				<td>ʬɽǡ̾Υ󥯸ʳΥ󥯸
				<td><input name="dr2.long.only_normal" value="false" type="radio"#{' checked'if not @setup['long.only_normal']}>ɽ
				<td><input name="dr2.long.only_normal" value="true" type="radio"#{' checked'if @setup['long.only_normal']}>
			<tr>
				<td>ǿɽǡ̾Υ󥯸ʳΥ󥯸
				<td><input name="dr2.short.only_normal" value="false" type="radio"#{' checked'if not @setup['short.only_normal']}>ɽ
				<td><input name="dr2.short.only_normal" value="true" type="radio"#{' checked'if @setup['short.only_normal']}>
			</table>
			<p>ǿɽǡ̾Υ󥯸ʳΥ󥯸ɽˤϡΥץ饰̵ȤޤäƱɽˤʤޤ</p>
			<h3>̾Υ󥯸Υ롼ײ</h3>
			<table>
			<tr>
				<td>̾Υ󥯸
				<td><input name="dr2.normal.group" value="true" type="radio"#{' checked'if @setup['normal.group']}>ִʸǤޤȤ
				<td><input name="dr2.normal.group" value="false" type="radio"#{' checked'if not @setup['normal.group']}>URLʬ롣
			<tr>
				<td>̾Υ󥯸ִʸǤޤȤˡǸ()
				<td><input name="dr2.normal.ignore_parenthesis" value="true" type="radio"#{' checked'if @setup['normal.ignore_parenthesis']}>̵뤹 /
				<td><input name="dr2.normal.ignore_parenthesis" value="false" type="radio"#{' checked'if not @setup['normal.ignore_parenthesis']}>̵뤷ʤ
			</table>
			<h3>ƥʤΥ󥯤Υ롼ײ</h3>
			<table>
			<tr>
				<td>ƥʤΥ󥯤
				<td><input name="dr2.antenna.group" value="true" type="radio"#{' checked'if @setup['antenna.group']}>ִʸǤޤȤ
				<td><input name="dr2.antenna.group" value="false" type="radio"#{' checked'if not @setup['antenna.group']}>URLʬ롣
			<tr>
				<td>ƥʤΥ󥯤ִʸǤޤȤˡǸ()
				<td><input name="dr2.antenna.ignore_parenthesis" value="true" type="radio"#{' checked'if @setup['antenna.ignore_parenthesis']}>̵뤹
				<td><input name="dr2.antenna.ignore_parenthesis" value="false" type="radio"#{' checked'if not @setup['antenna.ignore_parenthesis']}>̵뤷ʤ
			</table>
			<h3>ɤɽ</h3>
			<table>
			<tr>
				<td>󥸥̾
				<td><input name="dr2.search.expand" value="true" type="radio"#{' checked'if @setup['search.expand']}>ɽ
				<td><input name="dr2.search.expand" value="false" type="radio"#{' checked'if not @setup['search.expand']}>ɽʤ
			</table>
		_HTML
		unless @setup.secure then
		r << <<-_HTML
			<h3>å</h3>
			<table>
			<tr>
				<td>å
				<td><input name="dr2.no_cache" value="false" type="radio"#{' checked'if not @setup['no_cache']}>Ѥ
				<td><input name="dr2.no_cache" value="true" type="radio"#{' checked'if @setup['no_cache']}>Ѥʤ
			<tr>
				<td>ѹǡå
				<td><input name="dr2.cache.update" value="force" type="radio">
				<td><input name="dr2.cache.update" value="auto" type="radio" checked>ɬפʤ鹹
				<td><input name="dr2.cache.update" value="never" type="radio">ʤ
			</table>
			<p>
				åιˤ¿λ֤礬ޤ
				OKܥ򲡤餷Ф餯Ԥ
				å򹹿ʤɽ̷⤬뤳Ȥޤ
			</p>
		_HTML
		end # unless @setup.secure
		r
	end

	# shows URL list to be added to the referer_table or no_referer
	def show_unknown_list
		if @setup.secure then
			urls = DispRef2Latest_cache.unknown_urls
		elsif @setup['no_cache'] then
			urls = DispRef2Latest.new( @cgi, 'latest.rhtml', @conf, @setup ).unknown_urls
		else
			urls = DispRef2Cache.new( @setup ).unknown_urls
		end
		r = <<-_HTML
			<h3>󥯸ִꥹ</h3>
			<input name="dr2.current_mode" value="#{RefList}" type="hidden">
		_HTML
		if @cache then
			r << "<p>#{@setup['unknown.label']}ϥå椫õƤޤ"
		else
			r << "<p>#{@setup['unknown.label']}ϺǿɽõƤޤ"
		end
		r << <<-_HTML
			󥯸ꥹȤ̵ꥹȤ˰פURLϤˤɽޤ
		</p>
		<p>
			󥯸ִꥹȤ䵭ϿꥹȤˤ줿ʤURLϡ
			̵ꥹȤƤȤǡ
			ΥꥹȤ˸ʤʤޤ
			̵ꥹȤϡ
			ΥꥹȤURLɽ뤫ɤȽǤˤȤޤ
			<input name="dr2.clear_ignore_urls" value="true" type="checkbox">̵ꥹȤˤϥåƲ
		</p>
		_HTML
		if urls.size > 0 then
			r << <<-_HTML
				<p>󥯸ִꥹȤˤʤURL
					󥯸ִꥹȤϡ
					ʤζ˥ȥϤƤ
					ޤ󥯸ϿꥹȤɲäˤϡ
					åܥååƤ
				</p>
				<p>
					ɽϥ󥯸ִꥹȤɲäΤŬʤΤˤʤäƤޤ
					ǧơԶ礬ԽƤ
					󥯸ִꥹȤˤɲäˤϡ
					⤦ޥåξ郎ˤΤǤ⤫ޤޤ
				</p>
				<p>
					Ǹζϡ󥯸ִꥹȤɲäݤΥȥǤ
					URL˸줿()פϡ
					ִʸǡ\\1פΤ褦ʡֿפѤǤޤ
					ޤsprintf('[tdiary:%d]', $1.to_i+1) Ȥä
					ץҤѤǤޤ
				</p>
			_HTML
			if ENV['AUTH_TYPE'] and ENV['REMOTE_USER'] and @setup['configure.use_link'] then
				r << <<-_HTML
					<p>
						줾URLϥ󥯤ˤʤäƤޤ򥯥å뤳Ȥǡ
						ˡιѤURLΤ뤳Ȥˤʤޤ
						Ŭڤʥ¤̵ˤϥåʤ褦ˤƤ
					</p>
				_HTML
			end
			r << <<-_HTML
				<p>
					ˤʤURLϡ<a href="#{@conf.update}?conf=referer">󥯸</a>פ齤Ƥ
				</p>
				<dl>
			_HTML
			i = 0
			urls.sort.each do |url|
				shown_url = DispRef2String::escapeHTML( @setup.to_native( DispRef2String::unescape( url ) ) )
				if ENV['AUTH_TYPE'] and ENV['REMOTE_USER'] and @setup['configure.use_link'] then
					r << "<dt><a href=\"#{url}\">#{shown_url}</a>"
				else
					r << "<dt>#{shown_url}"
				end
				r << <<-_HTML
					<dd>
						<input name="dr2.#{i}.noref" value="true" type="checkbox">ꥹȤɲ
						<input name="dr2.#{i}.ignore" value="true" type="checkbox">̵ꥹȤɲ<br>
						<input name="dr2.#{i}.reg" value="#{DispRef2String::escapeHTML( DispRef2String::url_regexp( url ) )}" type="text" size="70"><br>
						<input name="dr2.#{i}.title" value="" type="text" size="70">
				_HTML
				i += 1
			end
			r << <<-_HTML
				<input name="dr2.urls" type="hidden" value="#{i}">
				</dl>
			_HTML
			unless @setup.secure or @setup['no_cache'] then
				r << <<-_HTML
					<p>
						åιˤ¿λ֤礬ޤ
						OKܥ򲡤餷Ф餯Ԥ
					</p>
				_HTML
			end
		else
			r << <<-_HTML
				<p>ߡ#{@setup['unknown.label']}Ϥޤ</p>
			_HTML
		end
		r << <<-_HTML
			<h3>ƥʤΤɽ</h3>
			<p>ƥʤURLִʸ˥ޥåɽǤ
				ɽ˥ޥå󥯸ϡ֥ƥʡפʬवޤ</p>
			<ul>
			<li>URL:
				<input name="dr2.antenna.url" value="#{DispRef2String::escapeHTML( @setup.to_native( @setup['antenna.url'] ) )}" type="text" size="70">
				<input name="dr2.antenna.url.default" value="true" type="checkbox">ǥեȤ᤹
			<li>ִʸ:<input name="dr2.antenna.title" value="#{DispRef2String::escapeHTML( @setup.to_native( @setup['antenna.title'] ) )}" type="text" size="70">
				<input name="dr2.antenna.title.default" value="true" type="checkbox">ǥեȤ᤹
			</ul>
			_HTML
		r
	end
end

# Hash table of search engines
# key: company name
# value: array of:
# [0]:url regexp [1]:title [2]:keys for search keyword [3]:cache regexp
DispReferrer2_Google_cache = /cache:[^:]+:([^+]+)+/
DispReferrer2_Yahoofs = /u=(.+)/
DispReferrer2_Engines = {
	'google' => [
		[%r{\Ahttp://(?:[^./]+\.)*?google\.([^/]+)/(search|custom|ie)}i, '".#{$1}Google"', ['as_q', 'q', 'as_epq'], DispReferrer2_Google_cache],
		[%r{\Ahttp://.*?\bgoogle\.([^/]+)/.*url}i, '".#{$1}GoogleURL?"', ['as_q', 'q'], DispReferrer2_Google_cache],
		[%r{\Ahttp://.*?\bgoogle/search}i, '"֤Google"', ['as_q', 'q'], DispReferrer2_Google_cache],
		[%r{\Ahttp://eval.google\.([^/]+)}i, '".#{$1}Google Accounts"', [], nil],
		[%r{\Ahttp://(?:[^./]+\.)*?google\.([^/]+)}i, '".#{$1}Google"', [], nil],
		[%r{\Ahttp://images\.google\.([^/]+)/imgres}i, '".#{$1}Google᡼"', ['imgurl'], DispReferrer2_Google_cache],
	],
	'yahoo' => [
		[%r{\Ahttp://.*?\.rd\.yahoo\.([^/]+)}i, '".#{$1}YahooΥ쥯"', 'split(/\*/)[1]', nil],
		[%r{\Ahttp://srd\.yahoo\.co\.jp}i, '"YahooΥ쥯"', [], nil],
		[%r{\Ahttp://rd.+\.yahoo\.com}i, '"YahooΥ쥯"', [], nil], # 󥸥 inktomi ȸ
		[%r{\Ahttp://.*?\.yahoo\.([^/]+)}i, '".#{$1}Yahoo!"', ['p', 'va', 'vp'], DispReferrer2_Google_cache],
	],
	'yahoofs' => [[%r{\Ahttp://cache\.yahoofs\.jp/}i, '"Yahoo!"', ['w'], DispReferrer2_Yahoofs]],
	'netscape' => [[%r{\Ahttp://.*?\.netscape\.([^/]+)}i, '".#{$1}Netscape"', ['search', 'query'], DispReferrer2_Google_cache]],
	'msn' => [[%r{\Ahttp://.*?\.MSN\.([^/]+)}i, '".#{$1}MSN"', ['q', 'MT'], nil ]],
	'metacrawler' => [[%r{\Ahttp://.*?.metacrawler.com}i, '"MetaCrawler"', ['q'], nil ]],
	'metabot' => [[%r{\Ahttp://.*?\.metabot\.ru}i, '"MetaBot.ru"', ['st'], nil ]],
	'altavista' => [[%r{\Ahttp://.*?\.altavista\.([^/]+)}i, '".#{$1}AltaVista"', ['q'], nil ]],
	'infoseek' => [[%r{\Ahttp://(www\.)?infoseek\.co\.jp}i, '"ե"', ['qt'], nil ]],
	'odn' => [[%r{\Ahttp://.*?\.odn\.ne\.jp}i, '"ODN"', ['QueryString', 'key'], nil ]],
	'lycos' => [[%r{\Ahttp://.*?\.lycos\.([^/]+)}i, '".#{$1}Lycos"', ['query', 'q', 'qt'], nil ]],
	'fresheye' => [[%r{\Ahttp://.*?\.fresheye}i, '"եå奢"', ['kw'], nil ]],
	'goo' => [
		[%r{\Ahttp://.*?\.goo\.ne\.jp}i, '"goo"', ['MT'], nil ],
		[%r{\Ahttp://.*?\.goo\.ne\.jp}i, '"goo"', [], nil ],
	],
	'nifty' => [
		[%r{\Ahttp://search\.nifty\.com}i, '"@nifty/@search"', ['q', 'Text'], DispReferrer2_Google_cache],
		[%r{\Ahttp://srchnavi\.nifty\.com}i, '"@niftyΥ쥯"', ['title'], nil ],
	],
	'eniro' => [[%r{\Ahttp://.*?\.eniro\.se}i, '"Eniro"', ['q'], DispReferrer2_Google_cache]],
	'excite' => [[%r{\Ahttp://.*?\.excite\.([^/]+)}i, '".#{$1}Excite"', ['search', 's', 'query', 'qkw'], nil ]],
	'biglobe' => [
		[%r{\Ahttp://.*?search\.biglobe\.ne\.jp}i, '"BIGLOBE"', ['q'], nil ],
		[%r{\Ahttp://.*?search\.biglobe\.ne\.jp}i, '"BIGLOBE"', [], nil ],
	],
	'dion' => [[%r{\Ahttp://dir\.dion\.ne\.jp}i, '"Dion"', ['QueryString', 'key'], nil ]],
	'naver' => [[%r{\Ahttp://.*?\.naver\.co\.jp}i, '"NAVER Japan"', ['query'], nil ]],
	'webcrawler' => [[%r{\Ahttp://.*?\.webcrawler\.com}i, '"WebCrawler"', ['qkw'], nil ]],
	'euroseek' => [[%r{\Ahttp://.*?\.euroseek\.com}i, '"Euroseek.com"', ['string'], nil ]],
	'aol' => [
		[%r{\Ahttp://.*?\.aol\.}i, '"AOL"', ['query'], nil ],
		[%r{\Ahttp://aolsearch\..+\.aol\.com/redir_convert.adp}i, '"AOL"', ['query_contain'], nil]
	],
	'alltheweb' => [
		[%r{\Ahttp://.*?\.alltheweb\.com}i, '"AlltheWeb.com"', ['q'], nil ],
		[%r{\Ahttp://.*?\.alltheweb\.com}i, '"AlltheWeb.com"', [], nil ],
	],
	'kobe-u' => [
		[%r{\Ahttp://bach\.scitec\.kobe-u\.ac\.jp/cgi-bin/metcha.cgi}i, '"å㸡󥸥"', ['q'], nil ],
		[%r{\Ahttp://bach\.istc\.kobe-u\.ac\.jp/cgi-bin/metcha.cgi}i, '"å㸡󥸥"', ['q'], nil ],
	],
	'tocc' => [[%r{\Ahttp://www\.tocc\.co\.jp/search/}i, '"TOCC/Search"', ['QRY'], nil ]],
	'yappo' => [[%r{\Ahttp://i\.yappo\.jp/}i, '"iYappo"', [], nil ]],
	'suomi24' => [[%r{\Ahttp://.*?\.suomi24\.([^/]+)/.*query}i, '"Suomi24"', ['q'], DispReferrer2_Google_cache]],
	'earthlink' => [[%r{\Ahttp://search\.earthlink\.net/search}i, '"EarthLink Search"', ['as_q', 'q', 'query'], DispReferrer2_Google_cache]],
	'infobee' => [[%r{\Ahttp://infobee\.ne\.jp/}i, '"󸡺"', ['MT'], nil ]],
	't-online' => [[%r{\Ahttp://brisbane\.t-online\.de/}i, '"T-Online"', ['q'], DispReferrer2_Google_cache]],
	'walla' => [[%r{\Ahttp://find\.walla\.co\.il/}i, '"Walla! Channels"', ['q'], nil ]],
	'mysearch' => [[%r{\Ahttp://.*?\.mysearch\.com/}i, '"My Search"', ['searchfor'], nil ]],
	'jword' => [[%r{\Ahttp://search\.jword.jp/}i, '"JWord"', ['name'], nil ]],
	'nytimes' => [[%r{\Ahttp://query\.nytimes\.com/search}i, '"New York Times: Search"', ['as_q', 'q', 'query'], DispReferrer2_Google_cache]],
	'aaacafe' => [[%r{\Ahttp://search\.aaacafe\.ne\.jp/search}i, '"AAA!CAFE"', ['key'], nil]],
	'virgilio' => [[%r{\Ahttp://search\.virgilio\.it/search}i, '"VIRGILIO Ricerca"', ['qs'], nil]],
	'ceek' => [[%r{\Ahttp://www\.ceek\.jp}i, '"ceek.jp"', ['q'], nil]],
	'cnn' => [[%r{\Ahttp://websearch\.cnn\.com}i, '"CNN.com"', ['query', 'as_q', 'q', 'as_epq'], DispReferrer2_Google_cache]],
	'webferret' => [[%r{\Ahttp://webferret\.search\.com}i, '"WebFerret"', 'split(/,/)[1]', nil]],
	'eniro' => [[%r{\Ahttp://www\.eniro\.se}i, '"Eniro"', ['query', 'as_q', 'q'], DispReferrer2_Google_cache]],
	'passagen' => [[%r{\Ahttp://search\.evreka\.passagen\.se}i, '"Eniro"', ['q', 'as_q', 'query'], DispReferrer2_Google_cache]],
	'redbox' => [[%r{\Ahttp://www\.redbox\.cz}i, '"RedBox"', ['srch'], nil]],
	'odin' => [[%r{\Ahttp://odin\.ingrid\.org}i, '"ODiN"', ['key'], nil]],
	'kensaku' => [[%r{\Ahttp://www\.kensaku\.}i, '"kensaku.jp"', ['key'], nil]],
	'hotbot' => [[%r{\Ahttp://www\.hotbot\.}i, '"HotBot Web Search"', ['MT'], nil ]],
	'searchalot' => [[%r{\Ahttp://www\.searchalot\.}i, '"Searchalot"', ['q'], nil ]],
	'cometsystems' => [[%r{\Ahttp://search\.cometsystems\.com}i, '"Comet Web Search"', ['qry'], nil ]],
	'bulkfeeds' => [
		[%r{\Ahttp://bulkfeeds\.net/app/search2}i, '"Bulkfeeds: RSS Directory & Search"', ['q'], nil ],
		[%r{\Ahttp://bulkfeeds\.net/app/similar}i, '"Bulkfeeds Similarity Search"', ['url'], nil ],
	],
	'answerbus' => [
		[%r{\Ahttp://www\.answerbus\.com}i, '"AnswerBus"', [], nil ],
		[%r{\Ahttp://answerbus\.coli\.uni-sb\.de/cgi-bin/answerbus/answer.cgi}i, '"AnswerBus"', [], nil ],
	],
	'dogplile' => [[%r{\Ahttp://www.\dogpile\.com/info\.dogpl/search/web/}i, '"dogpile"', [], nil ]],
	'www' => [[%r{\Ahttp://www\.google/search}i, '"Google?"', ['as_q', 'q'], DispReferrer2_Google_cache]],	# TLD missing
	'planet' => [[%r{\Ahttp://www\.planet\.nl/planet/}i, '"Planet-Zoekpagina"', ['googleq', 'keyword'], DispReferrer2_Google_cache]], # googleq parameter has a strange prefix
	'dcn' => [[%r{\Ahttp://www\.dcn\.to/~comment/cgi-bin/commenton\.cgi}i, '"᥿COMMENTON"', ['q'], nil ]],
	'ask' => [[%r{\Ahttp://ask\.jp/web.asp}i, '"ask.jp"', ['q'], nil ]],
	'searchscout' => [[%r{\Ahttp://results\.searchscout\.com/search}i, '"SearchSout"', ['k'], nil ]],
	'inktomi' => [[%r{\Ahttp://rdrw1.inktomi.com/click}i, '"inktomi Υ쥯"', [], nil]],
	'3721' => [[%r{\Ahttp://(seek|nmsearch)\.3721\.com/}i, '"3721Ӻ"', ['p','name'], nil]],
	'yisou' => [[%r{\Ahttp://www\.yisou\.com}i, '""', ['p'], nil]],
	'devilfinder' => [[%r{\Ahttp://www\.devilfinder\.com/find.php}i, '"The Devilfinder"', ['q'], nil]],
	'lyricsuniverse' => [[%r{\Ahttp://www\.lyricsuniverse\.com/}, '"LYRICS Universe"', [], nil]],
	'vivisimo' => [[%r{\Ahttp://.+\.vivisimo\.com/search}i, '"Vivisimo"', [], nil]],
	'a9' => [[%r{\Ahttp://a9\.com}i, '"A9"', [], nil]],
	'nttrd' => [[%r{\Ahttp://labs\.nttrd\.com/cgi-bin/index\.cgi}, '"goo"', ['q'], nil]],
	'whatis' => [[%r{\Ahttp://whatis\.techtarget\.com/wsearchResults/}i, '"WhatIs.com | web search"', ['query'], nil]],
	'comcast' => [[%r{\Ahttp://www\.comcast\.net/qry/websearch}i, '"COMCAST"', ['query'], nil]],
	'mywebsearch' => [[%r{\Ahttp://www\.mywebsearch\.com/jsp/GGmain.jsp}i, '"My Web Search"', ['seachfor'], nil]],
	'wisenut' => [[%r{\Ahttp://www\.wisenut\.com/search/query.dll}, '"WiseNut"', ['q'], nil]],
	'googlie' => [[%r{\Ahttp://www\.googlie\.com/search}i, '"Google(ؤΥ쥯)"', ['as_q', 'q'], DispReferrer2_Google_cache]],
	'toppg' => [[%r{\Ahttp://g\.toppg\.to/search}i, '"Google(ؤΥ쥯)"', ['as_q', 'q'], DispReferrer2_Google_cache]],
	'210' => [[%r{\Ahttp://210\.174\.160\.70/se_root.phtml}i, '"JWordܸ쥭ɡ"', ['name'], nil]],
	# % whois 64.233.160
	# NetRange:   64.233.160.0 - 64.233.191.255
	# CIDR:       64.233.160.0/19
	# NetName:    GOOGLE
	'233' => [[%r{\Ahttp://64\.233\.(1[6-8][0-9]|190|191)\.}i, '"Google"', ['as_q', 'q'], DispReferrer2_Google_cache]],
	# % whois 66.102.0.0
	# NetRange:   66.102.0.0 - 66.102.15.255
	# CIDR:       66.102.0.0/20
	# NetName:    GOOGLE-2
	'102' => [[%r{\Ahttp://66\.102\.([0-9]|1[0-5])\.}i, '"Google"', ['as_q', 'q'], DispReferrer2_Google_cache]],
	# other google candidates:
	'216' => [[%r{\Ahttp://(\d+\.){3}\d+/search}i, '"Google?"', ['as_q', 'q'], DispReferrer2_Google_cache]],		# cache servers of google?
	# NetRange:   72.14.192.0 - 72.14.255.255
	'14' => [[%r{\Ahttp://72\.14\.(19[2-9]|2\d\d)\.}i, '"Google"', ['as_q', 'q'], DispReferrer2_Google_cache]],
}
